#include <math.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdialog.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qtabwidget.h>
#include <qtable.h>
#include <qwidget.h>
#include <qwidgetfactory.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qspinbox.h>
#include <qtextedit.h>

#include "ui/dabout.h"
#include "ui/danimate.h"
#include "ui/darrneighbors.h"
#include "ui/darrowstyle.h"
#include "ui/datomstyle.h"
#include "ui/dcoordsys.h"
#include "ui/dprojvector.h"
#include "ui/dscalearrow.h"
#include "ui/dselectplots.h"
#include "ui/dselectzlayers.h"
#include "ui/drelaxinfo.h"
#include "ui/dscalepos.h"
#include "ui/dshowplanes.h"
#include "ui/dtransform.h"

#include "appwin.h"
#include "calcs.h"
#include "fformat.h"
#include "messages.h"
#include "misc.h"
#include "neighlist.h"
#include "pltwin.h"
#include "setatomstyle.h"
#include "ddplot.h"


void AppWin::about()
{
  DAbout *dialog = new DAbout( this, "", true, WShowModal );
  dialog->exec();
  delete( dialog );
}


void AppWin::animate()
{
  QString outf,cmd,args,animFile,msg;
  int n,nbefore;
  int NPlots = tabWidget->count();
  int density,delay,nloops;
  bool exit;


  // checking to see if ImageMagick is present
  exit = system("convert -version")==-1;
  if (exit) {
    msgError(err_NoImageMagick);    
    return;
  }

  DAnimate *dialog = new DAnimate( this, "", true, WShowModal );
  dialog->exec();
  
  if (!dialog->result()==QDialog::Accepted)
    return;

  animFile = dialog->animFile->text();
  density = dialog->density->value();
  delay = dialog->delay->value();
  nloops = dialog->nloops->value();
  args = dialog->args->text();

  // collecting the data

  system("if [ -e *.#af ]; then rm *.#af; fi");

  nbefore = tabWidget->currentPageIndex();
  printer->setColorMode(QPrinter::Color);

  QProgressDialog pdlg( "Collecting plots...", "&Abort", NPlots,
			this, "progress", TRUE );

  exit = false;

  for (n=0; n<NPlots; n++) {
    if (!plotWidget[n]->isSelected)
      continue;

    outf.sprintf( "%03d.#af", n);
    printer->setOutputToFile(true);
    printer->setOutputFileName( outf );
    
    tabWidget->setCurrentPage(n);
    plotWidget[n]->printIt(printer);

    pdlg.setProgress(n);
    if (pdlg.wasCancelled()) {  
      exit = true;
      n = NPlots-1;
    }
  }

  printer->setOutputToFile( false );
  tabWidget->setCurrentPage( nbefore );
  plotWidget[nbefore]->repaint();

  if (exit)
    return;

  // calling the ImageMagick

  pdlg.setLabelText("Generating animation... (this will take a while)");
  pdlg.setProgress(NPlots);

  cmd.sprintf("convert -density %dx%d -delay %d %s *.#af %s",
	      density,density,delay,args.latin1(),animFile.latin1());
  system(cmd.latin1());
  system("rm *.#af");

  msg.sprintf(info_AnimationDone,animFile.latin1());
  QMessageBox::information( 0, "Information", msg );
}


void AppWin::arrNeighbors()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QString txt;
  int i,istart,iend;
  bool ok;

  DArrNeighbors *dialog = new DArrNeighbors( this, "", true, WShowModal );

  // set the neighbor distances
  if (pw->RdfNNeigh>0) {
    txt.sprintf("R = %0.4f",pw->RdfDist[0]);
    dialog->Rdist1->setText(txt); 
  }
  if (pw->RdfNNeigh>1) {
    txt.sprintf("R = %0.4f",pw->RdfDist[1]);
    dialog->Rdist2->setText(txt); 
  }
  if (pw->RdfNNeigh>2) {
    txt.sprintf("R = %0.4f",pw->RdfDist[2]);
    dialog->Rdist3->setText(txt); 
  }
  if (pw->RdfNNeigh>3) {
    txt.sprintf("R = %0.4f",pw->RdfDist[3]);
    dialog->Rdist4->setText(txt); 
  }
  if (pw->RdfNNeigh>4) {
    txt.sprintf("R = %0.4f",pw->RdfDist[4]);
    dialog->Rdist5->setText(txt); 
  }

  // set the checkboxes
  for (i=0; i<pw->arrNeighNum; i++) {
    if (pw->arrNeighbors[i]==1)  dialog->Rcheck1->setChecked(1);
    if (pw->arrNeighbors[i]==2)  dialog->Rcheck2->setChecked(1);
    if (pw->arrNeighbors[i]==3)  dialog->Rcheck3->setChecked(1);
    if (pw->arrNeighbors[i]==4)  dialog->Rcheck4->setChecked(1);
    if (pw->arrNeighbors[i]==5)  dialog->Rcheck5->setChecked(1);
  }

  // execute the dialog
  dialog->exec();
  ok = dialog->result()==QDialog::Accepted;

  // change settings, if needed
  if (ok) {
    if (dialog->applyHow->currentItem()==0)
      istart = iend = tabWidget->currentPageIndex();
    else {
      istart = 0;
      iend = tabWidget->count()-1;
    }

    for (i=istart; i<=iend; i++) {
      pw = plotWidget[i];
      if (!pw->isSelected)
	continue;
      
      pw->arrNeighNum = 0;
      if (dialog->Rcheck1->isChecked()) {
	pw->arrNeighbors[pw->arrNeighNum] = 1;
	pw->arrNeighNum++;
      }
      if (dialog->Rcheck2->isChecked()) {
	pw->arrNeighbors[pw->arrNeighNum] = 2;
	pw->arrNeighNum++;
      }
      if (dialog->Rcheck3->isChecked()) {
	pw->arrNeighbors[pw->arrNeighNum] = 3;
	pw->arrNeighNum++;
      }
      if (dialog->Rcheck4->isChecked()) {
	pw->arrNeighbors[pw->arrNeighNum] = 4;
	pw->arrNeighNum++;
      }
      if (dialog->Rcheck5->isChecked()) {
	pw->arrNeighbors[pw->arrNeighNum] = 5;
	pw->arrNeighNum++;
      }

      BuildNeighborList( pw, pw->NeighListUnrel );
      pw->repaint();
    }
  }
}


void AppWin::arrowsPlot()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  int NPlots = tabWidget->count();
  int n;
  bool ok;

  DArrowStyle *dialog = new DArrowStyle( this, "", true, WShowModal );
  dialog->ithickness->setValue( pw->thicknessArrow );
  dialog->ishortest->setValue( pw->shortestArrow );

  dialog->exec();
  ok = dialog->result()==QDialog::Accepted;

  if (ok) {
    for (n=0; n<NPlots; n++) {
      pw = plotWidget[n];
      if (!pw->isSelected)
	continue;

      pw->thicknessArrow = dialog->ithickness->text().toInt();
      pw->shortestArrow = dialog->ishortest->text().toInt();
      pw->repaint();
    }
  }
}


void AppWin::close()
{
  int NCurrentPlot = tabWidget->currentPageIndex();
  int NPlots = tabWidget->count();
  int i;


  //  tabWidget->removePage( plotWidget[NCurrentPlot] );
  tabWidget->removePage( plotWidget[NCurrentPlot] );
  delete( plotWidget[NCurrentPlot] );

  for (i=NCurrentPlot; i<NPlots; i++)
    plotWidget[i] = plotWidget[i+1];

  // the deleted plot was the last one
  if (NCurrentPlot>=tabWidget->count())
    NCurrentPlot = tabWidget->count()-1;

  if (tabWidget->count()==0) {
    delete( tabWidget );
    tabWidget = NULL;
    enableMenuItems(false);
  } else {
    tabWidget->showPage( plotWidget[NCurrentPlot] );
    repaintStatusBar();
  }
}


void AppWin::closeAll()
{
  int n;
  int NPlots = tabWidget->count();

  for (n=0; n<NPlots; n++) {
    tabWidget->removePage( plotWidget[n] );
    delete( plotWidget[n] );
  }

  delete( tabWidget );
  tabWidget = NULL;

  repaintStatusBar();
  enableMenuItems(false);
}


bool AppWin::coordSys()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  bool ok;

  DCoordSys *dialog = new DCoordSys( this, "", true, WShowModal );

  dialog->x1->insert( QString::number( pw->csys[0][0] ));
  dialog->x2->insert( QString::number( pw->csys[0][1] ));
  dialog->x3->insert( QString::number( pw->csys[0][2] ));
  dialog->y1->insert( QString::number( pw->csys[1][0] ));
  dialog->y2->insert( QString::number( pw->csys[1][1] ));
  dialog->y3->insert( QString::number( pw->csys[1][2] ));
  dialog->z1->insert( QString::number( pw->csys[2][0] ));
  dialog->z2->insert( QString::number( pw->csys[2][1] ));
  dialog->z3->insert( QString::number( pw->csys[2][2] ));

  dialog->exec();
  ok = dialog->result()==QDialog::Accepted;

  if (ok) {
    pw->csys[0][0] = dialog->x1->text().toDouble();
    pw->csys[0][1] = dialog->x2->text().toDouble();
    pw->csys[0][2] = dialog->x3->text().toDouble();
    pw->csys[1][0] = dialog->y1->text().toDouble();
    pw->csys[1][1] = dialog->y2->text().toDouble();
    pw->csys[1][2] = dialog->y3->text().toDouble();
    pw->csys[2][0] = dialog->z1->text().toDouble();
    pw->csys[2][1] = dialog->z2->text().toDouble();
    pw->csys[2][2] = dialog->z3->text().toDouble();
  }
  
  return ok;
}


void AppWin::crystalPlot()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QPixmap pix;
  int NPlots = tabWidget->count();
  int i,n,nrows;
  bool ok;

  // declaration
  SetAtomStyle *astyle = new SetAtomStyle(pw);
  astyle->setData();

  // execution
  astyle->exec();
  ok = astyle->result()==QDialog::Accepted;

  // change of settings
  if (ok) {
    ok = astyle->verify();
    if (ok) {
      for (n=0; n<NPlots; n++) {
	pw = plotWidget[n];
	if (!pw->isSelected)  
	  continue;

	for (i=0; i<=pw->NumZLayers; i++) {
	  pw->zDiamLayer[i] = astyle->atomTable->text(i,1).toInt();
	  pw->zLineThickLayer[i] = astyle->atomTable->text(i,2).toInt();
	  
	  pw->zColorLayer[i][0] = astyle->zColor[i][0];
	  pw->zColorLayer[i][1] = astyle->zColor[i][1];
	}
	pw->repaint();    
      }
    }
  }
}


void AppWin::disloCenter()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  SHOW_DCENTER = !SHOW_DCENTER;
  mplot->setItemChecked( DCenterID, SHOW_DCENTER );
  pw->DisloCenter = SHOW_DCENTER; 
  pw->repaint();
  repaintStatusBar();
}


void AppWin::enableMenuItems( bool flag )
{
  mfile->setItemEnabled( SaveID, flag );
  mfile->setItemEnabled( CloseID, flag );
  mfile->setItemEnabled( CloseAllID, flag );
  mfile->setItemEnabled( PrintID, flag );
  mfile->setItemEnabled( ExportID, flag );
  mfile->setItemEnabled( ExpXYZID, flag );
  mfile->setItemEnabled( ExpCFGID, flag );
  mfile->setItemEnabled( AnimateID, flag );
    
  mview->setItemEnabled( ZoomInID, flag );
  mview->setItemEnabled( ZoomOutID, flag );
  mview->setItemEnabled( FitWinID, flag );
  mview->setItemEnabled( PanRightID, flag );
  mview->setItemEnabled( PanLeftID, flag );
  mview->setItemEnabled( PanUpID, flag );
  mview->setItemEnabled( PanDownID, flag );
  mview->setItemEnabled( PanFRightID, flag );
  mview->setItemEnabled( PanFLeftID, flag );
  mview->setItemEnabled( PanFUpID, flag );
  mview->setItemEnabled( PanFDownID, flag );

  mplot->setItemEnabled( SelPlotsID, flag );
  mplot->setItemEnabled( UnrelViewID, flag );
  mplot->setItemEnabled( RelViewID, flag );
  mplot->setItemEnabled( CompViewID, flag );
  mplot->setItemEnabled( MagDispID, flag );
  mplot->setItemEnabled( NumAtomID, flag );
  mplot->setItemEnabled( EdgeCompID, flag );
  mplot->setItemEnabled( ScrewCompID, flag );
  mplot->setItemEnabled( DAtLayersID, flag );
  mplot->setItemEnabled( DAtTypesID, flag );
  mplot->setItemEnabled( InertAtomID, flag );  
  mplot->setItemEnabled( DCenterID, flag );
  mplot->setItemEnabled( SPlanesID, flag );
  mplot->setItemEnabled( ShowGbID, flag );
  mplot->setItemEnabled( RelaxInfoID, flag ); 

  mlayers->setItemEnabled( ShowZLayID, flag );
  mlayers->setItemEnabled( SetZLayID, flag );

  mtran->setItemEnabled( CoordSysID, flag );
  mtran->setItemEnabled( XFlipID, flag );
  mtran->setItemEnabled( YFlipID, flag );
  mtran->setItemEnabled( ZFlipID, flag );
  mtran->setItemEnabled( TransformID, flag );  

  mappear->setItemEnabled( CrystPlotID, flag );
  mappear->setItemEnabled( ArrPlotID, flag );
  mappear->setItemEnabled( ScalePosID, flag );
  mappear->setItemEnabled( ProjVectID, flag );
  mappear->setItemEnabled( ScaleArrID, flag );
  mappear->setItemEnabled( ArrNeighID, flag );

}


void AppWin::exportCFG()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QString aname, cfgfile, msg;
  double amass, s[3];
  int i, j, d, atype;
  FILE *fcfg;

  cfgfile = CreateFName(pw->FName,".cfg");
  fcfg = fopen(cfgfile.latin1(), "w+");

  fprintf(fcfg,"Number of particles = %d\n\n",pw->NInit);
  fprintf(fcfg,"A = 10 Angstrom (basic length scale)\n\n");

  // supercell's first edge (x-size of the block)
  fprintf(fcfg,"H0(1,1) = %0.4f A\n",pw->blSize[0]);
  fprintf(fcfg,"H0(1,2) = 0.0 A\n");
  fprintf(fcfg,"H0(1,3) = 0.0 A\n\n");

  // supercell's second edge (y-size of the block)
  fprintf(fcfg,"H0(2,1) = 0.0 A\n");
  fprintf(fcfg,"H0(2,2) = %0.4f A\n",pw->blSize[1]);
  fprintf(fcfg,"H0(2,3) = 0.0 A\n\n");

  // supercell's third edge (z-size of the block)
  fprintf(fcfg,"H0(3,1) = 0.0 A\n");
  fprintf(fcfg,"H0(3,2) = 0.0 A\n");
  fprintf(fcfg,"H0(3,3) = %0.4f A\n\n",pw->blSize[2]);

  // coordinates of atoms - initial configuration
  for (i=0; i<pw->NInit; i++) {
    atype = pw->atomType[i];
    aname = ATOM_NAME[atype];
    amass = AtomicMass(aname.upper());

    // reduced coordinates (between 0 and 1)
    for (d=0; d<3; d++)
      s[d] = (pw->xyzInit[i][d] - pw->xyzMin[d]) / pw->blSize[d];

    fprintf(fcfg,"%0.4f  %2s  %6.4f  %6.4f  %6.4f  0  0  0\n",
	    amass, aname.latin1(), s[0], s[1], s[2]);
  }
  
  fclose(fcfg);

  msg = QString("Atomic structure successfully saved into\nfile ")+cfgfile;
  msgInfo(msg); 
}


void AppWin::exportXYZ()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QString aname, xyzfile, msg;
  FILE *f;
  int n,atype;

  xyzfile = CreateFName(pw->FName,".xyz");
  f = fopen(xyzfile.latin1(), "w+");

  fprintf(f,"%d\n",pw->NRel);
  fprintf(f,"Place a comment here\n");
  for (n=0; n<pw->NRel; n++) {
    atype = pw->atomType[n];
    aname = ATOM_NAME[atype];
    fprintf(f,"%s  %12.5lf  %12.5lf  %12.5lf\n",
      aname.latin1(),pw->xyzRel[n][0],pw->xyzRel[n][1],pw->xyzRel[n][2]);
  }

  fclose(f);

  msg = QString("Atomic structure successfully saved into\nfile ")+xyzfile;
  msgInfo(msg);  
}


void AppWin::fitWin()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->xPan = 0;
  pw->yPan = 0;
  pw->ZFact = 1;
  pw->repaint();

  repaintStatusBar();
}


void AppWin::flipBlock(int id)
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double mat[3][3], coor[3];
  int i, j, n;

  if (id==XFlipID) {
      mat[0][0] =  1;  mat[0][1] =  0;  mat[0][2] =  0;
      mat[1][0] =  0;  mat[1][1] = -1;  mat[1][2] =  0;
      mat[2][0] =  0;  mat[2][1] =  0;  mat[2][2] = -1;
      if (VERBOSE)
	printf("* the plot has been flipped about the X axis\n");
  } else if (id==YFlipID) {
      mat[0][0] = -1;  mat[0][1] =  0;  mat[0][2] =  0;
      mat[1][0] =  0;  mat[1][1] =  1;  mat[1][2] =  0;
      mat[2][0] =  0;  mat[2][1] =  0;  mat[2][2] = -1;
      if (VERBOSE)
	printf("* the plot has been flipped about the Y axis\n");
  } else if (id==ZFlipID) {
      mat[0][0] = -1;  mat[0][1] =  0;  mat[0][2] =  0;
      mat[1][0] =  0;  mat[1][1] = -1;  mat[1][2] =  0;
      mat[2][0] =  0;  mat[2][1] =  0;  mat[2][2] =  1;
      if (VERBOSE)
	printf("* the plot has been flipped about the Z axis\n");
  }

  // transformation of coordinates of atoms - unrelaxed
  for (n=0; n<pw->NInit; n++) {
    for (i=0; i<3; i++) {
      coor[i] = 0;
      for (j=0; j<3; j++) {
	coor[i] += mat[i][j]*pw->xyzInit[n][j];
      }
    }
    
    for (i=0; i<3; i++)
      pw->xyzInit[n][i] = coor[i];
  }
  
  // transformation of coordinates of atoms - relaxed
  for (n=0; n<pw->NRel; n++) {
    for (i=0; i<3; i++) {
      coor[i] = 0;
      for (j=0; j<3; j++) {
	coor[i] += mat[i][j]*pw->xyzRel[n][j];
      }
    }
    
    for (i=0; i<3; i++)
      pw->xyzRel[n][i] = coor[i];    
  }
  
  pw->PrepareBlock();
  pw->repaint();
}


void AppWin::interpretCommand(QString cmd)
{
  QString params[100];
  int npar,idx,i,j,id;
  int angle;
  bool alldone=false;

  // get rid of comments
  idx = cmd.find('#',0);   cmd.truncate(idx);
  idx = cmd.find('\n',0);  cmd.truncate(idx);
  idx = cmd.find('\r',0);  cmd.truncate(idx);  // Linux terminates with \r\n

  if (cmd.isEmpty())
    return;

  for (npar=0; npar<100; npar++) {
    params[npar] = cmd.section(',',npar,npar);
    if (params[npar].isEmpty())
      break;
    params[npar] = params[npar].stripWhiteSpace();    // get rid of spaces
  }

  if (npar<1)
    return;

  params[0] = params[0].upper();

  //-------------------------------------------
  // interpretation of the commands read above
  //-------------------------------------------

  if (params[0]=="ATOM.DIAMETER") {
    for (i=1; i<=npar-1; i++)
      ATOM_DIAM[i-1] = params[i].toInt();
    NUM_DIAM = npar-1;
  }

  if (params[0]=="ATOM.THICKNESS") {
    for (i=1; i<=npar-1; i++)
      ATOM_THICKNESS[i-1] = params[i].toInt();
    NUM_THICKNESS = npar-1;
  }

  if (params[0]=="ATOM.FGCOLOR") {
    for (i=1; i<=npar-1; i++)
      ATOM_FGCOLOR[i-1] = QColor(params[i].latin1());
    NUM_FGCOLORS = npar-1;
  }

  if (params[0]=="ATOM.BGCOLOR") {
    for (i=1; i<=npar-1; i++)
      ATOM_BGCOLOR[i-1] = QColor(params[i].latin1());
    NUM_BGCOLORS = npar-1;
  }

  if (params[0]=="ANAME") {
    id = params[1].toInt();
    ATOM_NAME[id] = params[2];
  }

  if (params[0]=="ATOMNUMBERS") {
    ATOM_NUMBERS = (params[1]=="on");
  }
      
  if (params[0]=="ARRNEIGHBORS") {
    ARR_NEIGHNUM = npar-1;
    for (i=1; i<=npar-1; i++)   
      ARR_NEIGHBORS[i-1] = params[i].toInt();
  }

  if (params[0]=="ADISTINGUISH") {
    if (params[1]=="layer")
      PLOT_TYPE = PLOT_ATOM_LAYERS;

    if (params[1]=="type")
      PLOT_TYPE = PLOT_ATOM_TYPES;
  }

  if (params[0]=="ARROW.SHORTEST")          
    ARROW_SHORTEST = params[1].toInt();

  if (params[0]=="ARROW.THICKNESS")          
    ARROW_THICKNESS = params[1].toInt();

  if (params[0]=="ATOMPOS") {
    if (params[1]=="relaxed")
      ATOMPOS = RELAXED;

    if (params[1]=="unrelaxed")
      ATOMPOS = UNRELAXED;

    if (params[1]=="composite")
      ATOMPOS = COMPOSITE;

    if (params[1]=="relaxed" or params[1]=="composite") {
      if (npar>2)
        ATOMDISP_SCALE = params[2].toInt();
      else
        ATOMDISP_SCALE = 1.0;
    }
  }
   
  if (params[0]=="CSYS") {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
        idx = 3*i+j+1;
        CSYS[i][j] = params[idx].toDouble();
      }
    }
  }

  if (params[0]=="DISPCOMPONENT") {
    if (params[1]=="edge")  
      DISP_COMPONENT = EDGE;

    if (params[1]=="screw")
      DISP_COMPONENT = SCREW;
  }

  if (params[0]=="DSCALEFACT") {
    if (params[1]=="auto")
      DSCALE_FACT = AUTO;
    else
      DSCALE_FACT = params[1].toDouble();
  }

  if (params[0]=="NEIGHRCUT") {
    NEIGHRCUT = params[1].toDouble();
  }

  if (params[0]=="PLANETRACES") {
    angle = params[1].toDouble(); 
    if (angle>0) {
      PTRACES = true; 
      PTRACES_ANGLE = angle;
    }
  }

  if (params[0]=="PRINTMAGDISP") {
    if (params[1]=="on")
      PRINT_MAG_DISP = true;

    else
      PRINT_MAG_DISP = false;
  }

  if (params[0]=="PROJVECTOR") {
    PROJ_VECTOR[0] = params[1].toDouble();
    PROJ_VECTOR[1] = params[2].toDouble();
    PROJ_VECTOR[2] = params[3].toDouble(); 
  }

  if (params[0]=="WINSIZE") {
    WINWIDTH = params[1].toInt();
    WINHEIGHT = params[2].toInt();
  }

  if (params[0]=="ZOOMFACT") {
    ZOOM_FACT = params[1].toDouble();
  }
}


bool AppWin::loadFile( QString fn )
{
  QString fname=fn;
  PltWin *initConf=NULL;
  long int blstart=0;
  int pos, NCurrentPlot, fmt, blnum=0;
  bool ok;


  if (tabWidget==NULL) {
    tabWidget = new QTabWidget( this, "tabWidget" );
    tabWidget->setGeometry( 0, 23, width(), height()-40 );  
  }

  // what is the format of the input file
  fmt = FFormat(fn);

  while (fmt!=FORMAT_GB || (fmt==FORMAT_GB && blstart>=0)) {   
    NCurrentPlot = tabWidget->count();
    if (NCurrentPlot>=MAX_PLOTS) {
      msgError(err_TooManyPlots);
      return false;
    }
    
    plotWidget[NCurrentPlot] = new PltWin( tabWidget );
    
    switch(fmt) {
    case FORMAT_DD:  // ddplot internal format
      plotWidget[NCurrentPlot]->LoadDD( fn );
      break;
      
    case FORMAT_BOP:  // format of BOP (see block.out)
      plotWidget[NCurrentPlot]->LoadBOP( fn );	   
      break;
      
    case FORMAT_PLT:  // plot format of BOP
      plotWidget[NCurrentPlot]->LoadPLT( fn );
      break;
      
    case FORMAT_GB:  // grain boundary data file (multiple blocks in one file)
      blnum++;
      ok = plotWidget[NCurrentPlot]->LoadGB( fn, &blstart, initConf );
      if (blnum==1)  initConf = plotWidget[NCurrentPlot];
      break;
      
    case FORMAT_UNKNOWN:
      msgError(err_InputFmtUnknown);     
    }
    
    plotWidget[NCurrentPlot]->PrepareBlock();
    plotWidget[NCurrentPlot]->SetGeometry();
    
    // only the file name is printed
    fname = fn;
    pos = fname.findRev('/');  
    if (pos>=0)
      fname.remove(0,pos+1);

    if (fmt==FORMAT_GB)
      fname.sprintf("%s (#%d)", fname.ascii(), blnum);
    
    tabWidget->addTab( plotWidget[NCurrentPlot], fname );
    tabWidget->showPage( plotWidget[NCurrentPlot] );
    tabWidget->show();

    if (fmt!=FORMAT_GB)  break;
  }

  repaintStatusBar();
  enableMenuItems(true);

  return true;
}


void AppWin::loadSettings()
{
  QString tbuf;
  FILE *f;
  char *buf;
  bool eof;

  f = fopen( SETTINGS_FILE, "r+" );
  if (f!=NULL) {
    if (VERBOSE)
      printf("* reading settings from file %s\n",SETTINGS_FILE.latin1());
  } else {
    if (VERBOSE)
      printf("* no configuration file ~/.ddplot found => default settings will be used\n");
    return;
  }

  buf = (char *) malloc(128);

  while (!feof(f)) {
    // read whole line
    eof = fgets(buf,128,f)==NULL;
    if (eof)
      break;

    tbuf = buf;
    interpretCommand(tbuf);
  }

  free(buf);
  fclose(f);
}


void AppWin::open()
{
  QString types( "Plot files (*.plt *.dat *.dd st.interm.*);;"
		 "BOP block files (*.bl *.block block.out block.interm);;"
		 "Any file (*)");
  int n;
  bool ok;

  QStringList files = QFileDialog::getOpenFileNames(
    types,
    "./",
    this,
    "Load plot file"
    "Select one or more files to open" );

  if (files.isEmpty()) return;

  QStringList::Iterator fn = files.begin();  
  QProgressDialog pdlg( "Loading plot files...", "&Abort", files.count(),
			this, "progress", TRUE);
  
  ok = true;
  n = 0;
  while( fn!=files.end() && ok ) {
    ok = loadFile(*fn);
    
    if (ok) {
      fn++;
      n++;
      pdlg.setProgress(n);
      pdlg.repaint();
      
      if (pdlg.wasCancelled())
	return;
    }
  }
}


void AppWin::panRight()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(PAN_NORMAL,0);
  repaintStatusBar();
}


void AppWin::panLeft()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(-PAN_NORMAL,0);
  repaintStatusBar();
}


void AppWin::panUp()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,PAN_NORMAL);
  repaintStatusBar();
}


void AppWin::panDown()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,-PAN_NORMAL);
  repaintStatusBar();
}


void AppWin::panFastRight()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(PAN_FAST,0);
  repaintStatusBar();
}


void AppWin::panFastLeft()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(-PAN_FAST,0);
  repaintStatusBar();
}


void AppWin::panFastUp()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,PAN_FAST);
  repaintStatusBar();
}


void AppWin::panFastDown()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,-PAN_FAST);
  repaintStatusBar();
}


void AppWin::plotAtomLayers()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->plotType = PLOT_ATOM_LAYERS;
  pw->repaint();

  mplot->setItemChecked( DAtLayersID, true );
  mplot->setItemChecked( DAtTypesID, false );
}


void AppWin::plotAtomTypes()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  
  pw->plotType = PLOT_ATOM_TYPES;
  pw->repaint();

  mplot->setItemChecked( DAtLayersID, false );
  mplot->setItemChecked( DAtTypesID, true );
}


void AppWin::plotEdgeComponent()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->DispComponent = EDGE;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( EdgeCompID, true);
  mplot->setItemChecked( ScrewCompID, false);
}

void AppWin::plotScrewComponent()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->DispComponent = SCREW;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( EdgeCompID, false);
  mplot->setItemChecked( ScrewCompID, true);
}


void AppWin::plotUnrelaxed()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = UNRELAXED;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, true );
  mplot->setItemChecked( RelViewID, false );
  mplot->setItemChecked( CompViewID, false );
}


void AppWin::plotRelaxed()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = RELAXED;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, false );
  mplot->setItemChecked( RelViewID, true );
  mplot->setItemChecked( CompViewID, false );
}


void AppWin::plotComposite()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = COMPOSITE;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, false );
  mplot->setItemChecked( RelViewID, false );
  mplot->setItemChecked( CompViewID, true );
}


void AppWin::plotInertAtoms()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->InertAtoms = (!pw->InertAtoms);
  plotRelaxed();  // the inert atoms belong to the relaxed configuration

  mplot->setItemChecked( UnrelViewID, false );
}


void AppWin::print()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  if (printer->setup(this))
    pw->printIt(printer);
}


void AppWin::printMagDisp()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->PrintMagDisp = abs(pw->PrintMagDisp-1); 
  pw->repaint();

  mplot->setItemChecked( MagDispID, pw->PrintMagDisp );
}


void AppWin::printAtomNum()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomNumbers = abs(pw->AtomNumbers-1); 
  pw->repaint();

  mplot->setItemChecked( NumAtomID, pw->AtomNumbers );
}


void AppWin::projVector()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double coord[3];
  int NPlots = tabWidget->count();
  int i,n;
  bool ok,okx,oky,okz;

  do {
    // dialog may not be WDestructiveClose, because it would delete all data inside exec()
    DProjVector *dialog = new DProjVector( this, "", true, WShowModal );

    dialog->xcoord->insert( QString::number( pw->ProjVector[0] ));
    dialog->ycoord->insert( QString::number( pw->ProjVector[1] ));
    dialog->zcoord->insert( QString::number( pw->ProjVector[2] ));

    dialog->exec();
    ok = dialog->result()==QDialog::Accepted;

    if (ok) {
      coord[0] = dialog->xcoord->text().toDouble( &okx );
      coord[1] = dialog->ycoord->text().toDouble( &oky );
      coord[2] = dialog->zcoord->text().toDouble( &okz );

      if (okx && oky && okz) {
	for (n=0; n<NPlots; n++) {
	  pw = plotWidget[n];
	  if (!pw->isSelected) 
	    continue;

	  for (i=0; i<3; i++)
	    pw->ProjVector[i] = coord[i];

	  pw->DispComponent = MIXED;
	  repaintStatusBar();
	  pw->repaint();
	}
      } else
        msgError(err_ProjVectIsString);
    }
  } while ( ok && (!okx || !oky || !okz) );
}


void AppWin::quit()
{
  //  if (msgWarning( warn_ReallyQuit ))
  exit(0);
}


void AppWin::relaxInfo()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  DRelaxInfo *dialog = new DRelaxInfo( this, "", true, WShowModal );

  if (pw->tail!=NULL) {
    dialog->textArea->append( pw->tail );
    dialog->textArea->setCursorPosition(0,0);
  }

  dialog->exec();
}


void AppWin::repaintStatusBar()
{
  if (tabWidget==NULL)
    return;

  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  char *dcomp;
  char msg[128];
  int NPlots = tabWidget->count();

  switch(pw->DispComponent) {
    case EDGE:
      dcomp = "EDGE";
      break;
    case SCREW:
      dcomp = "SCREW";
      break;
    default:
      dcomp = "MIXED";
  }

  sprintf(msg, "%d plot(s) | MAG=%0.2lf | DSCALE=%0.2lf | PANX=%0.2lf, PANY=%0.2lf" \
    " | PROJVECT=[%0.1lf,%0.1lf,%0.1lf] | DCOMP=%s | ATOMS=%s", 
    NPlots, 
    pw->ZFact,
    pw->DScaleFact,
    pw->xPan,
    pw->yPan,
    pw->ProjVector[0],
    pw->ProjVector[1],
    pw->ProjVector[2],
    dcomp,
    ATOMPOS_NAME[pw->AtomPos]);

  statusBar()->message( msg, 0 );
}


// Saves the current plot in the .dd format of ddplot. If we are
// saving a file which has been read in the .dd format, the coordinate
// system is not requested because it was read from the .dd file. If
// the original file is .plt, the system asks for the coordinate
// system of the block.
void AppWin::saveDD()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QString fname, msg;
  FILE *fdd;
  double sum;
  int i, j, pos;
  bool ok, needcsys;

  // Is the coordinate system of the block given ?
  needcsys = true;
  if (pw->FName.lower().findRev(".dd")>=0) {
    sum = 0;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	sum += fabs(pw->csys[i][j]);
      }
    }
    needcsys = (sum==0);
  }

  // Give the coordinate system    
  if (needcsys) {
    ok = coordSys();
    if (!ok)
      return;
  }

  fname = CreateFName(pw->FName,".dd");
  fdd = fopen(fname.latin1(),"w+");

  // header
  fprintf(fdd,"#\n");
  fprintf(fdd,"#  To view the atomic structure stored in this file, use ddplot\n");
  fprintf(fdd,"#  which is downloadable from http://www.lrsm.upenn.edu/~vitek\n");
  fprintf(fdd,"#\n\n");

  // coordinate system of the block
  fprintf(fdd,"CSYS\n");
  for (i=0; i<3; i++) {
    fprintf(fdd,"  %10.6lf  %10.6lf  %10.6lf\n",
	    pw->csys[i][0], pw->csys[i][1], pw->csys[i][2]); 
  }
  fprintf(fdd,"\n");

  // periodicity along x,y,z
  fprintf(fdd,"PERIOD\n");
  fprintf(fdd,"  %10.6lf  %10.6lf  %10.6lf\n\n",
	  pw->period[0], pw->period[1], pw->period[2]);

  // dislocation center
  fprintf(fdd,"DISLO_CENTER\n");
  fprintf(fdd,"  %10.6lf  %10.6lf\n\n",
	  pw->xCore, pw->yCore);

  // unrelaxed coordinates
  fprintf(fdd,"NUM_UNREL\n  %d\n\n",pw->NInit);
  fprintf(fdd,"COOR_UNREL\n");
  for (i=0; i<pw->NInit; i++) {
    fprintf(fdd,"  %10.6lf  %10.6lf  %10.6lf  %2d\n",
	    pw->xyzInit[i][0], pw->xyzInit[i][1], pw->xyzInit[i][2], pw->atomType[i]);
  }
  fprintf(fdd,"\n");

  // relaxed coordinates
  fprintf(fdd,"NUM_REL\n  %d\n\n",pw->NRel);
  fprintf(fdd,"COOR_REL\n");
  for (i=0; i<pw->NRel; i++) {
    fprintf(fdd,"  %10.6lf  %10.6lf  %10.6lf\n",
	    pw->xyzRel[i][0], pw->xyzRel[i][1], pw->xyzRel[i][2]);
  }
  fprintf(fdd,"\n");

  // details of the relaxation
  if (pw->tail!=NULL) {
    fprintf(fdd,"RELAX_DETAILS\n");
    fwrite(pw->tail,1,pw->TailSize-1,fdd);
  }
  
  
  fclose(fdd);

  msg = QString("Atomic structure successfully saved into\nfile ")+fname;
  msgInfo(msg);

  // change the name of the widget
  pos = fname.findRev('/');  
  if (pos>=0)
    fname.remove(0,pos+1);
  tabWidget->setTabLabel(pw,fname);
}


void AppWin::scaleArrow()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double sfact;
  int NPlots = tabWidget->count();
  int n;
  bool ok,okk;

  DScaleArrow *dialog = new DScaleArrow( this, "", true, WShowModal );
  dialog->sfactor->insert( QString::number( pw->DScaleFact ));
  do {
    dialog->exec();
    ok = dialog->result()==QDialog::Accepted;

    if (ok) {
      sfact = dialog->sfactor->text().toDouble( &okk );
      if (okk) {
	for (n=0; n<NPlots; n++) {
	  pw = plotWidget[n];
	  if (!pw->isSelected) 
	    continue;

	  pw->DScaleFact = sfact; 
	  pw->repaint();
	}
      } else
        msgError(err_ScaleArrowIsString);   
    }
  } while (ok && !okk);

  repaintStatusBar();
}


void AppWin::scalePos()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double fact;
  int sigAuto=100;
  int res, i, istart, iend;
  bool ok, okk;

  // create the dialog window
  DScalePos *dialog = new DScalePos( this, "", true, WShowModal );
  dialog->magRatio->insert( QString::number( pw->factor ));
  dialog->exec();
  
  // which button has been clicked ?
  res = dialog->result();
  
  if (res==QDialog::Accepted || res==sigAuto) {
    if (res==QDialog::Accepted) {
      fact = dialog->magRatio->text().toDouble( &ok );
      if (!ok) {
	msgError(err_ScalePosIsString);   
	return;
      }
    }

    // check if the magnification ratio should be applied to the current widget
    // or to all selected plots
    if (dialog->applyHow->currentItem()==0)
      istart = iend = tabWidget->currentPageIndex();
    else {
      istart = 0;
      iend = tabWidget->count()-1;
    }
    
    // apply the new magnification ratio
    for (i=istart; i<=iend; i++) {
      pw = plotWidget[i];
      if (!pw->isSelected)
	continue;

      if (res==QDialog::Accepted) 
	pw->factor = fact;
      else
	pw->SetGeometry(); 
    }
  }

  pw->repaint();
}


void AppWin::showGb()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  
  pw->showGB = !pw->showGB;
  mplot->setItemChecked( ShowGbID, pw->showGB );
  pw->repaint();
}


void AppWin::showPlanes()
{
  int i,astep;
  bool ok;

  DShowPlanes *dialog = new DShowPlanes( this, "", true, WShowModal );
  dialog->angle->setValue( PTRACES_ANGLE );
  dialog->exec();
  ok = dialog->result()==QDialog::Accepted;

  if (!ok)
    return;

  astep = dialog->angle->text().toDouble( &ok );
  PTRACES_ANGLE = astep;

  for (i=0; i<tabWidget->count(); i++) {
    if (!plotWidget[i]->isSelected) 
      continue;  
    
    if (astep>0) {
      plotWidget[i]->PlaneTraces = true;
      plotWidget[i]->PTAngle = astep;
    } else {
      plotWidget[i]->PlaneTraces = false;
    }
  }

  // only the current widget needs to be repainted
  plotWidget[ tabWidget->currentPageIndex() ]->repaint();
}


void AppWin::showZLayers()
{
  int i;

  SHOW_ZLAYERS = !SHOW_ZLAYERS;
  mlayers->setItemChecked( ShowZLayID, SHOW_ZLAYERS );

  // apply to all selected plots
  for (i=0; i<tabWidget->count(); i++) {
    if (plotWidget[i]->isSelected)
      plotWidget[i]->showZLayers = !plotWidget[i]->showZLayers;
  }
  
  plotWidget[ tabWidget->currentPageIndex() ]->repaint();
}


void AppWin::selectPlots()
{
  QString item;
  int sigSelAll=100, sigUnselAll=101;
  int pos,res;
  unsigned int i;
  bool exit,selFlag;


  DSelectPlots *dialog = new DSelectPlots( this, "", true, WShowModal );

  // set up the data
  for (i=0; i<tabWidget->count(); i++) {
    item = plotWidget[i]->FName;
    
    // only the file name is printed
    pos = item.findRev('/');  
    if (pos>=0)
      item.remove(0,pos+1);
    
    selFlag = plotWidget[i]->isSelected;

    dialog->plotFiles->insertItem(item);
    dialog->plotFiles->setSelected(i,selFlag);
  }

  do {
    // get the user's response
    dialog->exec();
    res = dialog->result();

    exit = true;

    // clicked on Select All
    if (res==sigSelAll) {
      for (i=0; i<dialog->plotFiles->count(); i++)
	dialog->plotFiles->setSelected(i,true);
      exit = false;
    }

    // clicked on Unselect All
    if (res==sigUnselAll) {
      for (i=0; i<dialog->plotFiles->count(); i++)
	dialog->plotFiles->setSelected(i,false);
      exit = false;
    }

    // clicked OK
    if (res==QDialog::Accepted) {
      for (i=0; i<dialog->plotFiles->count(); i++) {
	selFlag = dialog->plotFiles->isSelected(i);
	plotWidget[i]->isSelected = selFlag;
	tabWidget->setTabEnabled( plotWidget[i], selFlag );
      }

      tabWidget->repaint();
      repaintStatusBar();
    }
  } while (!exit);
}


void AppWin::selectZLayers()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QString item, stat;
  int sigSelAll=100, sigUnselAll=101;
  int pos, res, istart, iend, iplt;
  unsigned int i;
  bool exit, selFlag;


  DSelectZLayers *dialog = new DSelectZLayers( this, "", true, WShowModal );

  // set up the data
  for (i=0; i<pw->NumZLayers; i++) {
    item = QString::number( pw->zCoordLayer[i], 'f', 4 );
    selFlag = pw->zLayerSel[i];

    dialog->zLayers->insertItem(item);
    dialog->zLayers->setSelected(i,selFlag);
  }

  do {
    // get the user's response
    dialog->exec();
    res = dialog->result();

    exit = true;

    // clicked on Select All
    if (res==sigSelAll) {
      for (i=0; i<pw->NumZLayers; i++)
	dialog->zLayers->setSelected(i,true);
      exit = false;
    }

    // clicked on Unselect All
    if (res==sigUnselAll) {
      for (i=0; i<pw->NumZLayers; i++)
	dialog->zLayers->setSelected(i,false);
      exit = false;
    }

    // clicked OK
    if (res==QDialog::Accepted) {
      if (dialog->applyHow->currentItem()==0)
	istart = iend = tabWidget->currentPageIndex();
      else {
	istart = 0;
	iend = tabWidget->count()-1;
      }

      for (iplt=istart; iplt<=iend; iplt++) {
	PltWin *pw = plotWidget[iplt];

	for (i=0; i<pw->NumZLayers; i++) {
	  selFlag = dialog->zLayers->isSelected(i);
	  pw->zLayerSel[i] = selFlag;
	}

	if (VERBOSE) {
	  if (iend>istart)
	    printf("* working on plot #%d:  %s ...\n",iplt+1,pw->FName.ascii());
	  printf("* list of z-layers, their positions along the z-axis and visibility:\n");
	  printf("  (differential displacements will be calculated only between atoms in visible layers)\n");
	  
	  for (i=0; i<pw->NumZLayers; i++) {
	    if (pw->zLayerSel[i])
	      stat = "visible";
	    else
	      stat = "hidden";
	    
	    printf("    %3d  %8.4lf  %s\n", 
		   i+1, pw->zCoordLayer[i], stat.ascii());
	  }
	}

	// Construct the neighbor list in the unrelaxed configuration
	BuildNeighborList( pw, pw->NeighListUnrel );

	pw->repaint();
      }
    }
  } while (!exit);
}


void AppWin::transform()
{
  PltWin *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double newCSys[3][3], mat[3][3], coor[3], sum;
  int res, i, j, n, istart, iend, iplt;
  bool needcsys;


  // Is the coordinate system of the block given ?
  needcsys = true;
  sum = 0;
  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      sum += fabs(pw->csys[i][j]);
    }
  }
  needcsys = (sum==0);

  if (needcsys) {
    msgInfo(info_GiveCoordBlock);
    return;
  }

  DTransform *dialog = new DTransform( this, "", true, WShowModal );

  dialog->x1->setText( QString::number( pw->csys[0][0] ));
  dialog->x2->setText( QString::number( pw->csys[0][1] ));
  dialog->x3->setText( QString::number( pw->csys[0][2] ));
  dialog->y1->setText( QString::number( pw->csys[1][0] ));
  dialog->y2->setText( QString::number( pw->csys[1][1] ));
  dialog->y3->setText( QString::number( pw->csys[1][2] ));
  dialog->z1->setText( QString::number( pw->csys[2][0] ));
  dialog->z2->setText( QString::number( pw->csys[2][1] ));
  dialog->z3->setText( QString::number( pw->csys[2][2] ));
  
  dialog->exec();
  res = dialog->result();
  if (res!=QDialog::Accepted)
    return;

  // how to apply this settings?
  if (dialog->applyHow->currentItem()==0)
    istart = iend = tabWidget->currentPageIndex();
  else {
    istart = 0;
    iend = tabWidget->count()-1;
  }

  // transform the coordinates...
  for (iplt=istart; iplt<=iend; iplt++) {
    pw = plotWidget[iplt];
    if (!pw->isSelected)
      continue;

    newCSys[0][0] = dialog->X1->text().toDouble();
    newCSys[0][1] = dialog->X2->text().toDouble();
    newCSys[0][2] = dialog->X3->text().toDouble();
    newCSys[1][0] = dialog->Y1->text().toDouble();
    newCSys[1][1] = dialog->Y2->text().toDouble();
    newCSys[1][2] = dialog->Y3->text().toDouble();
    newCSys[2][0] = dialog->Z1->text().toDouble();
    newCSys[2][1] = dialog->Z2->text().toDouble();
    newCSys[2][2] = dialog->Z3->text().toDouble();
    
    // transformation matrix from pw->csys to newCSys
    mtransf(pw->csys,newCSys,mat);
    
    // transformation of coordinates of atoms - unrelaxed
    for (n=0; n<pw->NInit; n++) {
      for (i=0; i<3; i++) {
	coor[i] = 0;
	for (j=0; j<3; j++) {
	  coor[i] += mat[i][j]*pw->xyzInit[n][j];
	}
      }
      
      for (i=0; i<3; i++)
	pw->xyzInit[n][i] = coor[i];
    }
    
    // transformation of coordinates of atoms - relaxed
    for (n=0; n<pw->NRel; n++) {
      for (i=0; i<3; i++) {
	coor[i] = 0;
	for (j=0; j<3; j++) {
	  coor[i] += mat[i][j]*pw->xyzRel[n][j];
	}
      }
      
      for (i=0; i<3; i++)
	pw->xyzRel[n][i] = coor[i];    
    }
    
    pw->PrepareBlock();
    pw->repaint();
  }
}


void AppWin::zoomIn()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Zoom(2.0); 
  repaintStatusBar();
}


void AppWin::zoomOut()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Zoom(0.5); 
  repaintStatusBar();
}
