#include <math.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <stdlib.h>

#include "pltwin.h"


// Plotting of the edge component of the displacements
void PltWin::plotEdgeComponent(QPainter *p)
{
  QString txt;
  double arr_between[MAX_ATOMS][MAX_NEIGHBORS];
  double rdisp[3];
  double xy_from[2],xy_to[2],xfrom,yfrom,xto,yto;
  double cen,lproj,pvsize,ndisp,cdisp,len;
  int i,d,in,ineigh,icen,idx;
  bool exist, plotted;

  for (i=0; i<NInit; i++)
    arr_between[i][0] = END_OF_LIST;

  // plotting of arrows
  for (icen=0; icen<NInit; icen++) {
    // plot only those atoms which belong to the active layers
    if (!zLayerSel[zLayer[icen]])
      continue;

    idx = in = 0;    

    while (NeighListUnrel[icen][idx]!=END_OF_LIST) {
      ineigh = NeighListUnrel[icen][idx];

      // plot only those atoms which belong to the active layers
      if (!zLayerSel[zLayer[ineigh]]) {
	idx++;
	continue;
      }

      // does the arrow between icen<-ineigh already exist ?
      i = 0;
      exist = false;
      while (arr_between[ineigh][i]!=END_OF_LIST) {
        if (arr_between[ineigh][i]==icen) {
          exist = true;
          break;
        }
        i++;
      }

      // yes, the arrow exists - get out
      if (exist) {
        idx++;
        continue;
      }
   
      // arrow does not exist - plot it
      arr_between[icen][in] = ineigh;                    // the arrow is between icen and ineigh
      in++;

      //----- from here it holds only for edges and mixed dislocations -----      
      switch(DispComponent) {
        case EDGE:
          for (d=0; d<2; d++)
            rdisp[d] = aDisp[ineigh][d]-aDisp[icen][d];      // relative displacement of the two atoms
          break;

        case MIXED:
          for (d=0; d<3; d++)
            rdisp[d] = aDisp[ineigh][d]-aDisp[icen][d];

	  // dot product of rdisp with ProjVector (normalized)
	  lproj = 0;
          pvsize = sqrt( pow(ProjVector[0],2)+pow(ProjVector[1],2)+pow(ProjVector[2],2) );
          for (d=0; d<3; d++)
            lproj += rdisp[d]*ProjVector[d]/pvsize; 
 
          for (d=0; d<3; d++)
            rdisp[d] = lproj*ProjVector[d];
      }

      // The arrow must always point in the direction of larger total
      // displacement. If the displacement of the neighbor atom is
      // smaller than that of the central atom, the arrow will be
      // plotted in the opposite direction. 
      ndisp = sqrt(pow(aDisp[ineigh][0],2)+pow(aDisp[ineigh][1],2));
      cdisp = sqrt(pow(aDisp[icen][0],2)+pow(aDisp[icen][1],2));

      if (ndisp<cdisp) {
	rdisp[0] = -rdisp[0];
	rdisp[1] = -rdisp[1];
      }
     
      // Adjusting the length of the arrow. If rdisp is greater or
      // equal than the periodicity/2.0 in the corresponding
      // direction, we take rdisp = rdisp modulo period/2.0
      for (d=0; d<=2; d++) { 
	if (rdisp[d]>=period[d]/2.0 || rdisp[d]<=-period[d]/2.0) {
	  rdisp[d] -= period[d]*trunc(rdisp[d]/(period[d]/2.0));
	  rdisp[d] *= 2.0;  // keep the size
        }
      }

      // calculation of the length of an arrow between icen and ineigh
      for (d=0; d<2; d++) {
	if (AtomPos==UNRELAXED)
          cen = (xyzInit[icen][d]+xyzInit[ineigh][d])/2.0;         // center of the arrow
        else
          cen = (xyzRel[icen][d]+xyzRel[ineigh][d])/2.0;           // center of the arrow

        xy_from[d] = cen - rdisp[d]/2.0*DScaleFact;
        xy_to[d] = cen + rdisp[d]/2.0*DScaleFact;
      }
      //--------------------------------------------------------------------

      // plotting of the arrows
      xfrom = ZFact*xy_from[0]-xPan-(ZFact-1)*xyzCen[0];
      yfrom = ZFact*xy_from[1]-yPan-(ZFact-1)*xyzCen[1];
      xto = ZFact*xy_to[0]-xPan-(ZFact-1)*xyzCen[0];
      yto = ZFact*xy_to[1]-yPan-(ZFact-1)*xyzCen[1];

      plotted = DrawArrow(p,xfrom,yfrom,xto,yto);

      // annotations
      if (PrintMagDisp && plotted) {
	len = sqrt( pow(rdisp[0],2) + pow(rdisp[1],2) );
        txt.sprintf("a/%0.1lf",1.0/fabs(len));
        DrawText( p, (xfrom+xto)/2.0, (yfrom+yto)/2.0, txt );
      }

      idx++;
    }

    arr_between[icen][in] = END_OF_LIST;               // terminator
  }
}


// Plotting of the screw component of the displacements
void PltWin::plotScrewComponent(QPainter *p)
{
  QString txt;
  double arr_between[MAX_ATOMS][MAX_NEIGHBORS];
  double rdisp[3],xyz0[3],xyz1[3],rvect[3];
  double xyz_from[3],xyz_to[3];
  double rscreen[3];
  double rproj,ratio,cen;
  int i,d,in,ineigh,icen,idx;
  bool exist,plotted;


  for (i=0; i<NInit; i++)
    arr_between[i][0] = END_OF_LIST;

  // plotting of arrows
  for (icen=0; icen<NInit; icen++) {
    // plot only those atoms which belong to the active layers
    if (!zLayerSel[zLayer[icen]])
      continue;

    idx = in = 0;    

    while (NeighListUnrel[icen][idx]!=END_OF_LIST) {
      ineigh = NeighListUnrel[icen][idx];

      // plot only those atoms which belong to the active layers
      if (!zLayerSel[zLayer[ineigh]]) {
	idx++;
	continue;
      }

      // does the arrow between icen<-ineigh already exist ?
      i = 0;
      exist = false;
      while (arr_between[ineigh][i]!=END_OF_LIST) {
        if (arr_between[ineigh][i]==icen) {
          exist = true;
          break;
        }
        i++;
      }

      // yes, the arrow exists - get out
      if (exist) {
        idx++;
        continue;
      }
   
      // arrow does not exist - plot it
      arr_between[icen][in] = ineigh;                    // the arrow is between icen and ineigh
      in++;
      
      for (d=0; d<3; d++)
        rdisp[d] = aDisp[ineigh][d]-aDisp[icen][d];      // relative displacement of the two atoms

      //----- from here it holds only for screws -----
      rproj = rdisp[2];                                   // z-displacement
      ratio = rproj/lenBurgers;
      
      if (fabs(ratio)>1.0)                                // treatment if ratio>1
        ratio = ratio-int(ratio);      

      if (fabs(ratio)>0.5)
        ratio -= fabs(ratio)/ratio;

      // calculation of the length of an arrow between icen and ineigh
      if (AtomPos==UNRELAXED) {
        for (d=0; d<3; d++) {
          xyz0[d] = xyzInit[icen][d];
          xyz1[d] = xyzInit[ineigh][d]; 
        }
      } else {
        for (d=0; d<3; d++) {
          xyz0[d] = xyzRel[icen][d] + AtomDispScale*aDisp[icen][d];
          xyz1[d] = xyzRel[ineigh][d] + AtomDispScale*aDisp[ineigh][d]; 
        }
      }
      for (d=0; d<3; d++) {
        rvect[d] = xyz1[d]-xyz0[d];
	rscreen[d] = ratio*2*DScaleFact*rvect[d];            // length of the arrow (from -1 to 1)
      }

      // adjustment of arrows - they must lie between two atoms
      for (d=0; d<3; d++) {
        cen = (xyz0[d]+xyz1[d])/2.0;
        xyz_from[d] = cen - rscreen[d]/2.0;
        xyz_to[d] = cen + rscreen[d]/2.0;
      }
      //----------------------------------------------

      // plotting of the arrows
      plotted = DrawArrow( p, ZFact*xyz_from[0]-xPan-(ZFact-1)*xyzCen[0], ZFact*xyz_from[1]-yPan-(ZFact-1)*xyzCen[1], 
        ZFact*xyz_to[0]-xPan-(ZFact-1)*xyzCen[0], ZFact*xyz_to[1]-yPan-(ZFact-1)*xyzCen[1]);

      // annotations
      if (PrintMagDisp && plotted) {
        txt.sprintf("a/%0.1lf",1.0/(fabs(ratio)/2.0));
        DrawText( p, ZFact*(xyz_from[0]+xyz_to[0])/2.0, ZFact*(xyz_from[1]+xyz_to[1])/2.0, txt );
      }
 
      idx++;
    }

    arr_between[icen][in] = END_OF_LIST;            // terminator
  }
}


//---------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------


bool PltWin::DrawArrow( QPainter *p, double x0, double y0, double x1, double y1 )
{
  QPointArray pts;
  const double h = 0.2*tan(arrAngle*M_PI/360.0);
  double arrow[4][2] = { {0.0,0.0}, {1.0,0.0}, {0.8,h}, {0.8,-h} };
  int i,xy[4][2];
  double ang,xrot,yrot,size;

  size = sqrt(pow(x1-x0,2)+pow(y1-y0,2));
  if (factor*size<shortestArrow) 
    return false;

  // scaling and rotation
  ang = atan2(y1-y0,x1-x0);
  for (i=0; i<4; i++) {
    // scaling
    arrow[i][0] *= size;
    arrow[i][1] *= size;

    // rotation
    xrot = arrow[i][0]*cos(ang)-arrow[i][1]*sin(ang);
    yrot = arrow[i][0]*sin(ang)+arrow[i][1]*cos(ang);
    arrow[i][0] = xrot;
    arrow[i][1] = yrot;
  }

  for (i=0; i<4; i++) {
    xy[i][0] = xBorder + xOffset + (int) ceil(factor*(x0+arrow[i][0]-xyzMin[0]));
    xy[i][1] = height() - (yBorder + yOffset + (int) ceil(factor*(y0+arrow[i][1]-xyzMin[1])));
  }

  p->drawLine(xy[0][0],xy[0][1],xy[1][0],xy[1][1]);
  pts.setPoints(3,xy[1][0],xy[1][1],xy[2][0],xy[2][1],xy[3][0],xy[3][1]);
  p->drawPolygon( pts );
  return true;
}


void PltWin::DrawCircle( QPainter *p, int atom, double x, double y, int dia)
{
  int xpos,ypos;
  int rad = int(dia/2.0);

  // (0,0) is at the bottom left corner of the screen
  xpos = xBorder + xOffset + (int) floor(factor*(x-xyzMin[0])) - rad; 
  ypos = height() - (yBorder + yOffset + (int) floor(factor*(y-xyzMin[1]))) - rad;
  
  p->drawEllipse( xpos, ypos, dia, dia );

  // if the real atomic number is given, keep its coordinates on the screen
  if (atom>=0) {
    atomScreen[atom][0] = xpos + rad;
    atomScreen[atom][1] = ypos + rad;
  }
}


void PltWin::DrawLine( QPainter *p, double x0, double y0, double x1, double y1, int addx, int addy)
{
  int xfrom,xto,yfrom,yto;

  xfrom = xBorder + xOffset + (int) floor(factor*(x0-xyzMin[0])) + addx;
  xto   = xBorder + xOffset + (int) floor(factor*(x1-xyzMin[0])) + addx;
  yfrom = height() - (yBorder + yOffset + (int) floor(factor*(y0-xyzMin[1])) + addy);
  yto   = height() - (yBorder + yOffset + (int) floor(factor*(y1-xyzMin[1])) + addy);

  p->drawLine( xfrom, yfrom, xto, yto );
}


void PltWin::DrawPlaneTraces( QPainter *p, double x0, double y0, double x1, double y1 )
{
  int ang,circDia;
  int x0int,y0int,x1int,y1int;
  double xEnd,yEnd,xC,yC,rang;

  QPen *apen = new QPen();
  apen->setColor( darkGray );
  p->setPen(*apen);

  x0int = xBorder + xOffset + (int) floor(factor*(x0-xyzMin[0])); 
  y0int = height() - (yBorder + yOffset + (int) floor(factor*(y0-xyzMin[1])));  
  x1int = xBorder + xOffset + (int) floor(factor*(x1-xyzMin[0])); 
  y1int = height() - (yBorder + yOffset + (int) floor(factor*(y1-xyzMin[1])));

  circDia = 10;
  p->setClipRect(x0int-2*circDia, y1int-2*circDia,
		 x1int-x0int+4*circDia, y0int-y1int+4*circDia);

  xC = ZFact*xCore-xPan-(ZFact-1)*xyzCen[0];
  yC = ZFact*yCore-yPan-(ZFact-1)*xyzCen[1];

  for (ang=0; ang<360; ang+=PTAngle) {
    rang = ang*M_PI/180.0;
    xEnd = xCore + 100.0*cos(rang);
    yEnd = yCore + 100.0*sin(rang);

    DrawLine(p,xC,yC,xEnd,yEnd,0,0);
  }

  p->setClipping(false);
}


void PltWin::DrawText( QPainter *p, double x, double y, QString txt )
{
  int xf,yf;

  xf = xBorder + xOffset + (int) floor(factor*(x-xyzMin[0]));
  yf = height() - (yBorder + yOffset + (int) floor(factor*(y-xyzMin[1])));

  p->drawText( xf, yf, txt );
}


void PltWin::Pan( double xsteps, double ysteps )
{
  xPan += PAN_RATIO*xsteps*blSize[0];   
  yPan += PAN_RATIO*ysteps*blSize[1];   
  repaint();
}


void PltWin::SetGeometry()
{
  double xfact,yfact;

  xfact = (width()-2*xBorder)/blSize[0];
  yfact = (height()-2*yBorder)/blSize[1];
  factor = (xfact<yfact) ? xfact : yfact;

  xOffset = (int) floor( (width()-2*xBorder-factor*blSize[0])/2.0 );
  yOffset = (int) floor( (height()-2*yBorder-factor*blSize[1])/2.0 );
}


void PltWin::ShowActiveZLayers( QPainter *p )
{
  QString txt;
  double step, pos;
  int i, yheight;

  p->setPen( Qt::darkGray ); 
  p->setBrush( Qt::white );

  yheight = 12*(NumZLayers-1);
  if (yheight>height()-80)
    yheight = height()-80;

  p->drawRect( 10, height()-yheight-60, 130, yheight+40 );

  step = double(yheight)/(NumZLayers-1);
  for (i=0; i<NumZLayers; i++ ) {
    pos = height()-40 - i*step;

    // active layers in green, inactive in black
    if (zLayerSel[i])
      p->setPen( Qt::green );
    else
      p->setPen( Qt::black );

    p->drawLine( 75, pos, 120, pos );

    txt = txt.setNum(zCoordLayer[i],'f',4);
    p->drawText( 30, pos+step/2.0, txt );
  }
}


void PltWin::Zoom( double zfact )
{
  if (ZFact*zfact<=20 && ZFact*zfact>=0.1) {
    ZFact *= zfact;
    repaint();
  }
}
