#include <math.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <stdlib.h>

#include "appwin.h"
#include "calcs.h"
#include "messages.h"
#include "pltwin.h"
#include "ddplot.h"



PltWin::PltWin(QWidget *parent, const char *name, WFlags f) 
  : QWidget( parent, name, f )
{
  int d,i,j;

  xBorder = yBorder = 30;
  xOffset = yOffset = 0;
  arrAngle = 60.0;
  ZFact = ZOOM_FACT;
  xPan = yPan = 0;
  isSelected = true;
  DisloCenter = SHOW_DCENTER;
  PlaneTraces = PTRACES;
  PTAngle = PTRACES_ANGLE;

  tail = NULL;
  TailSize = 0;

  arrNeighNum = ARR_NEIGHNUM;
  for (i=0; i<arrNeighNum; i++)
    arrNeighbors[i] = ARR_NEIGHBORS[i];

  for (d=0; d<3; d++) 
    ProjVector[d] = PROJ_VECTOR[d];

  for (i=0; i<3; i++) 
    for (j=0; j<3; j++)
      csys[i][j] = CSYS[i][j];

  DScaleFact = DSCALE_FACT;
  shortestArrow = ARROW_SHORTEST;     // shortest arrow to be plotted
  thicknessArrow = ARROW_THICKNESS;
  plotType = PLOT_TYPE;
  DispComponent = DISP_COMPONENT;
  AtomPos = ATOMPOS;
  AtomDispScale = ATOMDISP_SCALE;
  PrintMagDisp = PRINT_MAG_DISP;
  AtomNumbers = ATOM_NUMBERS;

  InertAtoms = false;
  isGBfile = false;
  showGB = SHOWGB;
  showZLayers = SHOW_ZLAYERS;

  rcut = 0;

  setGeometry( 0, 23, parent->width(), parent->height()-23 );
}


PltWin::~PltWin()
{
  free(tail);
}


void PltWin::paintEvent( QPaintEvent *)
{
  QPainter *p = new QPainter(this);
  drawIt(p);
  p->end();
}


void PltWin::resizeEvent( QResizeEvent * )
{
  SetGeometry();
}


void PltWin::drawIt( QPainter *p )
{
  QString txt;
  double x, y, x0, y0, x1, y1;
  int n, layer, prev_layer, circDia, i, j;

  QPen *pen = new QPen();
  QBrush *brush = new QBrush();
  QBrush *brush_unrld = new QBrush();


  // plot the grain boundary line
  if (isGBfile && showGB) {
    x0 = ZFact*xyzMin[0] - xPan - (ZFact-1)*xyzCen[0];
    x1 = ZFact*xyzMax[0] - xPan - (ZFact-1)*xyzCen[0];
    y  = ZFact*gbYcoor - yPan - (ZFact-1)*xyzCen[1];
    DrawLine(p,x0,y,x1,y,0,0);
  }

  if (AtomPos==COMPOSITE) {
    brush_unrld->setColor( lightGray );
    brush_unrld->setStyle( SolidPattern );
  }

  // plotting of atomic arrangement in the initial configuration
  prev_layer = -1;
  x0 = y0 = x1 = y1 = 0;


  //
  //  Plot the atomic configuration
  //

  for (n=0; n<NInit; n++) {
    // plot only those atoms which belong to the Z-layers that are selected (active)
    if (!zLayerSel[zLayer[n]])
      continue;

    if (plotType==PLOT_ATOM_LAYERS)
      layer = zLayer[n]; 
    else
      layer = atomType[n];

    if (layer!=prev_layer) {
      pen->setColor( zColorLayer[layer][0] );
      pen->setWidth( zLineThickLayer[layer] );
      p->setPen( *pen );

      brush->setColor( zColorLayer[layer][1] );
      brush->setStyle( SolidPattern );
      p->setBrush( *brush );   

      circDia = zDiamLayer[layer];
      prev_layer = layer;
    }

    // plot the atomic positions

    switch(AtomPos) {
      case UNRELAXED:
        x = ZFact*xyzInit[n][0]-xPan-(ZFact-1)*xyzCen[0];
	y = ZFact*xyzInit[n][1]-yPan-(ZFact-1)*xyzCen[1];
        DrawCircle(p,n,x,y,circDia);
        break;

      case RELAXED:
	x = ZFact*(xyzInit[n][0]+AtomDispScale*aDisp[n][0])-xPan-(ZFact-1)*xyzCen[0];
        y = ZFact*(xyzInit[n][1]+AtomDispScale*aDisp[n][1])-yPan-(ZFact-1)*xyzCen[1];
        DrawCircle(p,n,x,y,circDia);
	break;

      case COMPOSITE:
	p->setBrush( *brush_unrld );
	x = ZFact*xyzInit[n][0]-xPan-(ZFact-1)*xyzCen[0];
	y = ZFact*xyzInit[n][1]-yPan-(ZFact-1)*xyzCen[1];
        DrawCircle(p,n,x,y,circDia);
  
        p->setBrush( *brush );
	x = ZFact*(xyzInit[n][0]+AtomDispScale*aDisp[n][0])-xPan-(ZFact-1)*xyzCen[0];
	y = ZFact*(xyzInit[n][1]+AtomDispScale*aDisp[n][1])-yPan-(ZFact-1)*xyzCen[1]; 
        DrawCircle(p,-1,x,y,circDia);
    }

    if (x<x0) x0 = x;
    if (x>x1) x1 = x;
    if (y<y0) y0 = y;
    if (y>y1) y1 = y;

    // atom numbers
    if (AtomNumbers) {
      txt.sprintf("%d",n+1);
      DrawText(p,x,y,txt);
    }
  }

  // plot the inert atoms
  if (InertAtoms) {
    pen->setColor( black );
    pen->setWidth( zLineThickLayer[layer] );
    p->setPen( *pen );

    brush->setColor( black );
    brush->setStyle( NoBrush );
    p->setBrush( *brush ); 
        
    for (n=0; n<NInert; n++) {
      // plot only those atoms which belong to the selected (active) layers
      if (!zLayerSel[zLayer[n]])
	continue;

      x = ZFact*xyzInert[n][0]-xPan-(ZFact-1)*xyzCen[0];
      y = ZFact*xyzInert[n][1]-yPan-(ZFact-1)*xyzCen[1];
      DrawCircle(p,n,x,y,circDia);
    }
  }

  // plotting the coordinate system centered at the initial position of
  // the dislocation line
  if (PlaneTraces) 
    DrawPlaneTraces(p,x0,y0,x1,y1);

  // plotting of arrows
  QPen apen( black, thicknessArrow );
  p->setPen( apen ); 
  p->setBrush( Qt::black );
   
  switch(DispComponent) {
    case EDGE:
      plotEdgeComponent(p);
      break;
    case SCREW:
      plotScrewComponent(p);
      break;
    case MIXED:
      plotEdgeComponent(p);
  }

  // position of dislocation center
  if (DisloCenter) {
    x = ZFact*xCore-xPan-(ZFact-1)*xyzCen[0];
    y = ZFact*yCore-yPan-(ZFact-1)*xyzCen[1];
    DrawCircle(p,-1,x,y,circDia);
  }

  // small inset to show the active Z-layers
  if (showZLayers)
    ShowActiveZLayers(p);

  /* lines showing the division of the block into cells (for the linked neighbor list)
  for (i=0; i<=ncell[0]; i++) {
    x = ZFact*(xyzMin[0]+i*cellsize[0]) - xPan - (ZFact-1)*xyzCen[0];
    y0 = ZFact*xyzMin[1] - yPan - (ZFact-1)*xyzCen[1];
    y1 = ZFact*xyzMax[1] - yPan - (ZFact-1)*xyzCen[1];
    DrawLine(p,x,y0,x,y1,0,0);    

    for (j=0; j<=ncell[1]; j++) {
      y = ZFact*(xyzMin[1]+j*cellsize[1]) - yPan - (ZFact-1)*xyzCen[1];
      x0 = ZFact*xyzMin[0] - xPan - (ZFact-1)*xyzCen[0];
      x1 = ZFact*xyzMax[0] - xPan - (ZFact-1)*xyzCen[0];
      DrawLine(p,x0,y,x1,y,0,0);
    }
  }
  */  
}


void PltWin::printIt( QPrinter *printer )
{
  QPainter *prn = new QPainter(printer);
  QString cmd, fn, qbuf;
  FILE *f, *f2;
  int pos, x, y, w, h;
  char *buf;
  int err;

  drawIt(prn);
  prn->end();

  // fix the bounding box                                                                  
  if (printer->outputToFile()) {
#ifdef Q_WS_X11  // X11 specific                                             
    buf = (char *) malloc(128);

    fn = printer->outputFileName();
    system("if [ -e bbox.tmp ]; then rm bbox.tmp; fi");
    cmd = "gs -sDEVICE=bbox -dNOPAUSE -dQUIET -dBATCH " + fn + " &> bbox.tmp";
    system(cmd.ascii());

    // read the bounding box using ghostscript
    f = fopen("bbox.tmp","r+");
    fscanf(f, "%s %d %d %d %d\n", buf, &x, &y, &w, &h);
    fclose(f);

    // rewrite the bounding box in the .ps file
    f = fopen(fn.ascii(), "r");
    f2 = fopen("out.tmp","w+");

    while (!feof(f)) {
      fgets(buf, 128, f);
      qbuf = buf;

      if (qbuf.find("%%BoundingBox:")>=0)
	fprintf(f2, "%%%%BoundingBox: %d %d %d %d\n", x, y, w, h);
      else
	fputs(buf, f2);
    }

    fclose(f2);
    fclose(f);

    system("rm bbox.tmp");
    cmd = "mv out.tmp " + fn;
    system(cmd.ascii());

    free(buf);

#elif defined(Q_WS_MACX)   // MacOS X specific                                             

#endif
  }
}

