#include <qcolordialog.h>
#include <qcombobox.h>
#include <qlayout.h>

#include "appwin.h"
#include "pltwin.h"
#include "setatomstyle.h"


SetAtomStyle::SetAtomStyle( PltWin *pwid )
{
  pw = pwid;

  // additional SLOT connections
  connect( atomTable, SIGNAL( clicked(int,int,int,const QPoint&) ),
           this, SLOT( setColor(int,int) ) );
}


void SetAtomStyle::setData(void)
{
  int i,nrows;

  // enough rows ?

  nrows = atomTable->numRows();
  if ( pw->NumZLayers>nrows )
    atomTable->insertRows( nrows, pw->NumZLayers-nrows );

  // settings

  QRect rect = atomTable->cellRect( 0, 3 );
  QPixmap pix( rect.width(), rect.height() );

  for ( i=0; i<pw->NumZLayers; i++ ) {
    atomTable->setText( i, 0, QString("%1").arg( pw->zCoordLayer[i], 0, 'f', 5 ));
    atomTable->setText( i, 1, QString("%1").arg( pw->zDiamLayer[i] ));
    atomTable->setText( i, 2, QString("%1").arg( pw->zLineThickLayer[i] ));

    QColor fgcolor = pw->zColorLayer[i][0];
    pix.fill( fgcolor );
    atomTable->setPixmap( i, 3, pix );    
    zColor[i][0] = fgcolor;

    QColor bgcolor = pw->zColorLayer[i][1];
    pix.fill( bgcolor );
    atomTable->setPixmap( i, 4, pix );
    zColor[i][1] = bgcolor;
  }
}


bool SetAtomStyle::verify()
{
  int diam,thick,i; 
  bool ok,ok_diam,ok_thick;

  ok = true;

  for (i=0; i<atomTable->numRows(); i++) {
    diam = atomTable->text(i,1).toInt( &ok_diam );
    thick = atomTable->text(i,2).toInt( &ok_thick );

    ok = ok || (ok_diam && ok_thick);
    if (!ok)
      return false;
  }

  return true;
}


//------------------------------------------------------------------------------
// SLOTS
//------------------------------------------------------------------------------

void SetAtomStyle::setColor( int row, int col )
{
  if (col==3 || col==4) {
    QColor color = QColorDialog::getColor( white, this );

    if ( color.isValid() ) { 
      QRect rect = atomTable->cellRect( 0, col );
      QPixmap pix( rect.width(), rect.height() );      
      pix.fill( color );
      atomTable->setPixmap( row, col, pix ); 
      zColor[row][col-3] = color;
    }
  }
}
